@@ -1,6 +1,6 @@
-## ----------------------------------------------------------------------------|
+## ------------------------------------------------------------------------------------------------|
 # Data generator according to the graph structure
-## ----------------------------------------------------------------------------|
+## ------------------------------------------------------------------------------------------------|
 bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian", 
 						prob = 0.2, size = NULL, mean = 0, class = NULL, 
 						cut = 4, b = 3, D = diag(p), K = NULL, sigma = NULL, 
@@ -17,7 +17,7 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 	    graph <- "fixed"
     } 
 	
-#--- build the graph structure ------------------------------------------------|
+#--- build the graph structure --------------------------------------------------------------------|
 	if( graph == "random" )
 	{
 		G <- matrix( 0, p, p )
@@ -130,6 +130,14 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 		G = matrix( 0, p, p )
 		resultGraph = .C( "scale_free", G = as.integer(G), as.integer(p), PACKAGE = "BDgraph" )
 		G = matrix( resultGraph $ G, p, p ) 
+		
+		j = sample( 1:p, 1 )
+		
+		for( i in ( c( 1:p )[ -j ] ) )
+		{
+			 G[ i, j ] = 1
+			 G[ j, i ] = 1
+		}
 	}
 	
 	if( graph == "AR1" )
@@ -157,7 +165,7 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 		K[ ( 2 : p ), 1 ] <- 0.1
 	}
 
-#--- generate multivariate data according to the graph structure --------------|
+#--- generate multivariate data according to the graph structure ----------------------------------|
 	if( n != 0 )
 	{
 		if( !is.null( sigma ) ) K <- solve( sigma )   
@@ -184,14 +192,14 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 		diag( G ) <- 0
 		p         <- nrow( G )
 		
-#--- generate multivariate normal data ----------------------------------------|
+#--- generate multivariate normal data ------------------------------------------------------------|
 		if( typeof( mean ) == "double" ) mean <- rep( mean, p )
 		R <- chol( sigma )
 		z <- matrix( rnorm( p * n ), p, n )
 		d <- t( R ) %*% z + mean
 		d <- t( d )
 
-#--- generate multivariate mixed data -----------------------------------------|
+#--- generate multivariate mixed data -------------------------------------------------------------|
 		if( type == "mixed" )
 		{
 			# generating mixed data which are 'count', 'ordinal', 'non-Gaussian', 
@@ -219,7 +227,7 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 			d[,col_number] <- qbinom( p = prob, size = 1, prob = 0.5 )
 		}
 
-#--- generate multivariate continuous non-Gaussian data -----------------------|
+#--- generate multivariate continuous non-Gaussian data -------------------------------------------|
 		if( type == "non-Gaussian" )
 		{
 			# generating multivariate continuous non-Gaussian data  
@@ -227,7 +235,7 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 			d    <- qexp( p = prob, rate = 10 )
 		}
 
-#--- generate multivariate discrete data --------------------------------------|
+#--- generate multivariate discrete data ----------------------------------------------------------|
 		if( type == "discrete" )
 		{
 			runif_m   <- matrix( runif( cut * p ), nrow = p, ncol = cut )   
@@ -242,9 +250,21 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 			
 			d = d - 1
 		}
+
+		if( type == "binary" )
+		{
+			if( p > 16 ) stop( "For type 'binary', number of nodes (p) must be less than 16" )
+			
+			## Generate clique factors
+			clique_factors = generate_clique_factors( ug = G )
+
+			d = sample_ug( n = n, ug = G, clique_factors = clique_factors )
+			
+			d = d - 1
+		}
 	}
 	
-#--- graph visualization ------------------------------------------------------|
+#--- graph visualization --------------------------------------------------------------------------|
 	if( vis )
 	{
 		graphG <- graph.adjacency( G, mode = "undirected", diag = FALSE )
@@ -254,7 +274,7 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 		             vertex.color = "white", vertex.size = size, vertex.label.color = 'black' )
 	}
 	
-#--- Saving the result --------------------------------------------------------|
+#--- Saving the result ----------------------------------------------------------------------------|
 	if( n != 0 )
 	{
 		simulation <- list( G = G, data = d, sigma = sigma, K = K, graph = graph, type = type )
@@ -266,7 +286,7 @@ bdgraph.sim = function( p = 10, graph = "random", n = 0, type = "Gaussian",
 	return( simulation )
 }
     
-## ----------------------------------------------------------------------------|
+## ------------------------------------------------------------------------------------------------|
 # Print function for simulation data
 print.sim = function( x, ... )
 {
@@ -290,7 +310,7 @@ print.sim = function( x, ... )
 	}
 }
     
-## ----------------------------------------------------------------------------|
+## ------------------------------------------------------------------------------------------------|
 # plot for class "sim" from bdgraph.sim function
 plot.sim = function( x, main = NULL, layout = layout.circle, ... )
 {
@@ -301,3 +321,81 @@ plot.sim = function( x, main = NULL, layout = layout.circle, ... )
     plot.igraph( g_igraph, main = main, layout = layout, ... )
 }		
        
+## ------------------------------------------------------------------------------------------------|
+# Function for exact sampling from binary data
+## ------------------------------------------------------------------------------------------------|
+sample_ug = function( n = 1, ug = diag( 3 ), clique_factors = NULL )
+{
+	p = ncol( ug ) # p smaller than 17 check
+	if( p > 16 ) stop( "number of nodes must be smaller than 16" )
+	ug[ lower.tri( ug, diag = TRUE ) ] = 0
+	if( is.null( clique_factors ) ) clique_factors = generate_clique_factors( ug )
+	
+	prob = calc_joint_dist( ug, clique_factors )
+	noc  = length( prob )
+	ind  = sample( 1:noc, n, replace = TRUE, prob = prob )
+	oc   = sapply( 1:noc, function( x ){ as.integer( intToBits( x ) ) } )
+	oc   = 2 - t( oc[ 1:p, ] )
+	data = oc[ ind, ]
+	
+	return( data )
+}
+    
+## ------------------------------------------------------------------------------------------------|
+generate_clique_factors = function( ug )
+{
+	ug[ lower.tri( ug, diag = TRUE ) ] = 0   
+	p              = ncol( ug )
+	edges          = which( ug == 1, arr.ind = T )
+	a              = make_undirected_graph( c( t( edges ) ), p )
+	cliques        = max_cliques( a )
+	clique_factors = vector( 'list', length( cliques ) )
+	
+	for ( i in 1:length( cliques ) )
+	{
+		clq                 = cliques[[i]]
+		clique_factors[[i]] = runif( 2 ^ length( clq ) )
+	}
+	
+	return( clique_factors )
+}
+    
+## ------------------------------------------------------------------------------------------------|
+calc_joint_dist = function( ug, clique_factors )
+{
+	p          = ncol( ug )
+	oc         = sapply( 1:( 2 ^ p ), function( x ){ as.integer( intToBits( x ) ) } )
+	oc         = 2 - t( oc[ 1:p, ] )
+
+	edges      = which( ug == 1, arr.ind = T )
+	a          = make_undirected_graph( c( t( edges ) ), p )
+
+	joint_dist = rep( 1, 2 ^ p )
+	cliques    = max_cliques( a )
+	
+	for ( i in 1:length( cliques ) )
+	{
+		clq        = cliques[[i]]
+		k          = length( clq )
+		temp       = sapply( 1:( 2 ^ k ), function( x ){ as.integer( intToBits( x ) ) } )
+		clq_oc     = 2 - t( temp[1:k, ] )
+		clq_factor = clique_factors[[i]]
+		
+		for ( j in 1:nrow( clq_oc ) )
+		{
+			oc_col_clq = oc[ , clq ]
+			if( is.null( dim( oc_col_clq ) ) ) 
+				oc_col_clq = matrix( oc_col_clq, nrow = length( oc_col_clq ), ncol = 1 )
+		
+			ind             = apply( oc_col_clq, 1, identical, clq_oc[ j, ] )
+			joint_dist[ind] = joint_dist[ ind ] * clq_factor[ j ]
+		}				
+	}	
+	
+	joint_dist = joint_dist / sum( joint_dist )
+	
+	return( joint_dist )	
+}
+    
+
+
