@@ -14,18 +14,8 @@
 #ifndef copula_H
 #define copula_H
 
-#include <R.h>
-#include <Rmath.h>
-#include <R_ext/Lapack.h>
-#include <R_ext/BLAS.h>
-#include <R_ext/Utils.h>
-#include <sstream>
-#include <string>        // std::string, std::to_string
-#include <vector>        // for using vector
 #include "matrix.h"
 
-using namespace std;
-
 extern "C" {
 	void get_mean( double Z[], double K[], double *mu_ij, double *sigma, int *i, int *j, int *n, int *p );
 
